import { DisposableMetadata, HookableMetadata } from "@surface/core";
import AsyncObserver from "../reactivity/async-observer.js";
export default function listener(...propertiesOrPaths) {
    return (target, propertyKey) => {
        const paths = propertiesOrPaths.map(x => Array.isArray(x) ? x : [x]);
        const finisher = (instance) => {
            const disposableMetadata = DisposableMetadata.from(instance);
            const method = Reflect.get(instance, propertyKey);
            for (const path of paths) {
                const observer = AsyncObserver.observe(instance, path);
                const subscription = observer.subscribe(x => method(x));
                disposableMetadata.add({ dispose: () => subscription.unsubscribe() });
            }
        };
        HookableMetadata.from(target.constructor).finishers.push(finisher);
    };
}
