import { enumerateRange } from "../common.js";
import { disposeTree } from "../singletons.js";
const BLOCKS = Symbol("custom-element:template-blocks");
export default class TemplateBlock {
    constructor() {
        this.disposed = false;
        this.open = document.createComment("#open");
        this.close = document.createComment("#close");
        this.open[BLOCKS] = new Set([this]);
        this.close[BLOCKS] = new Set([this]);
    }
    isAnchor(node) {
        return !!node[BLOCKS];
    }
    disconnect() {
        if (this.open[BLOCKS].size == 1) {
            this.open.remove();
        }
        else {
            this.open[BLOCKS].delete(this);
        }
        if (this.close[BLOCKS].size == 1) {
            this.close.remove();
        }
        else {
            this.close[BLOCKS].delete(this);
        }
    }
    optimize() {
        const hasNestedDirective = this.open.nextSibling
            && this.open.nextSibling != this.close
            && this.isAnchor(this.open.nextSibling)
            && this.close.previousSibling
            && this.close.previousSibling != this.open
            && this.isAnchor(this.close.previousSibling);
        if (hasNestedDirective) {
            const nextOpen = this.open.nextSibling;
            const previousClose = this.close.previousSibling;
            const open = this.open;
            const close = this.close;
            for (const block of open[BLOCKS].values()) {
                block.open = nextOpen;
                nextOpen[BLOCKS].add(block);
            }
            for (const block of close[BLOCKS].values()) {
                block.close = previousClose;
                previousClose[BLOCKS].add(block);
            }
            open.remove();
            close.remove();
        }
    }
    connect(node) {
        node.appendChild(this.open);
        node.appendChild(this.close);
    }
    clear() {
        for (const element of enumerateRange(this.open, this.close)) {
            element.remove();
            disposeTree(element);
        }
    }
    dispose() {
        if (!this.disposed) {
            this.clear();
            this.disconnect();
            this.disposed = true;
        }
    }
    insertAt(parent, reference) {
        parent.replaceChild(this.close, reference);
        parent.insertBefore(this.open, this.close);
    }
    setContent(content, optimize = true) {
        this.close.parentNode.insertBefore(content, this.close);
        if (optimize) {
            this.optimize();
        }
    }
}
